#!%PERL%
#
# ruboard %VERSION% common subroutines
#
# Copyright (c) 1998 by Andrew Maltsev, <am@amsoft.ru>
#
# $Id: rublib.PL,v 1.1 1998/09/11 07:30:58 am Exp $
#
# To alter supported languages search for @LANG@. Only russian and
# english languages supported now.
###############################################################################

###############################################################################
# Reading configuration. If we have `config' form parameter defined,
# then search for configuration file is done in following order:
#  1) $homedir/$config.cf
#  2) $homedir/$config/ruboard.cf
#  3) $homedir/ruboard.cf
# Only first word (as \w specifies) from `config' used.
#
sub read_config
{ undef $cf_file;
  if($FORM{"config"})
   { $FORM{"config"} =~ s/^\s*(\w+).*$/$1/;
     $config=$FORM{"config"};
     if($config)
      { if(-r "$homedir/$config.cf")
         { $cf_file="$homedir/$config.cf";
         }
        else
         { $cf_file="$homedir/$config/ruboard.cf";
           $basedir=$config;
           undef $cf_file unless -r $cf_file;
         }
      }
   }
  $cf_file="$homedir/ruboard.cf" unless $cf_file;

  # Setting defaults
  #
  $basedir=$config;
  $title="ruboard v%VERSION%";
  $cgi_url="http://" . $ENV{"HTTP_HOST"} . $ENV{"SCRIPT_NAME"};
  $admin_url="http://" . $ENV{"HTTP_HOST"} . $ENV{"SCRIPT_NAME"};
  $mesgdir="messages";
  $datafile="msgseq.dat";
  $passwd_file="passwd.dat";
  $ext="html";
  $mainpage="ruboard.$ext";
  $faqfile="faq.$ext";
  $language="english";
  $show_faq=0;
  $show_refs=1;
  $allow_html=0;
  $quote_text=1;
  $subject_line=0;
  $use_time=1;
  $bg_paper="#FFFFFF";		# Common background
  $bg_image="";			# Background image
  $bg_subject="#CCCCCC";	# Color of Subject line
  $bg_sep="#EEEEEE";		# Color of separators
  $cite_symbol=">";
  $max_entries=0;

  # Reading configuration
  #
  &error("no_config") unless -r $cf_file;
  do $cf_file;
  &error("no_config") if $@;

  # Fixing configuration
  #
  &error("wrong_cf","No base URL (\$baseurl) defined!") if !$baseurl;
  $basedir=$config if !$basedir;
  $basedir="$homedir/$basedir" unless $basedir =~ "^/";
  $datafile="$basedir/$datafile" unless $datafile =~ "^/";
  $passwd_file="$basedir/$passwd_file" unless $passwd_file =~ "^/";
  $messages=$mesgdir if !$messages;
  $messages="$basedir/$messages" unless $messages =~ "^/";
  mkdir($messages,0755) unless -d $messages;
  &error("wrong_cf","Bad path to messages directory ($messages)") unless -d $messages && -w _;
  $mesgfile=$mainpage if !$mesgfile;
  $mesgfile="$basedir/$mesgfile" unless $mesgfile =~ "^/";
  chop($baseurl) while $baseurl =~ "/$";
  $mainpage="$baseurl/$mainpage" unless $mainpage =~ "tp://";
  $cgi_url="$baseurl/$cgi_url" unless $cgi_url =~ "tp://";
  $admin_url="$baseurl/$admin_url" unless $admin_url =~ "tp://";
  $faqfile="$baseurl/$faqfile" unless $faqfile =~ "tp://";
  chop($cite_symbol) while $cite_symbol =~ /\s$/;
  $max_entries=scalar($max_entries);
  $bg_image="$baseurl/$bg_image" unless $bg_image =~ "tp://" || !$bg_image;

# Fields names
#@LANG@#
#
  if($language eq "russian")
   { $t_name="";
     $t_email="E-Mail";
     $t_subject="";
     $t_link_url=" (URL)";
     $t_link_text=" ";
     $t_image=" (URL)";
     $t_post="        ";
     $t_reset="        ";
     $t_followups="";
     $t_post_followup=" ";
     $t_faq="";
     $t_author="";
   }
  else
   { $t_name="Name";
     $t_email="E-Mail";
     $t_subject="Subject";
     $t_link_url="Optional link URL";
     $t_link_text="Link title";
     $t_image="Optional image URL";
     $t_post="    Post Message    ";
     $t_reset="    Reset    ";
     $t_followups="Follow Ups";
     $t_post_followup="Post Followup";
     $t_faq="FAQ";
     $t_author="Author";
   }
}

####################################
# Parse Form Subroutine
#
sub parse_form
{ # Get the input
  #
  if($ENV{"REQUEST_METHOD"} eq "GET")
   { $buffer=$ENV{"QUERY_STRING"};
   }
  else
   { read(STDIN, $buffer, $ENV{'CONTENT_LENGTH'});
   }

  # Split the name-value pairs
  #
  @pairs = split(/&/, $buffer);
  foreach $pair (@pairs)
   { ($nm, @_) = split(/=/, $pair);
     $value=join('=',@_);

     # Un-Webify plus signs and %-encoding
     $value =~ tr/+/ /;
     $value =~ s/%([a-f0-9][a-f0-9])/pack("C", hex($1))/eig;

     $FORM{$nm} = $value;
   }
}

###############################################################################
# Various common html tags
#
sub print_html_hr	{ print "<hr width=80%>\n"; }

###############################################################
# Converting special symbols to their HTML representations
#
sub to_html_text
{ local($_)=$_[0];
  s/&/&amp;/g;
  s/</&lt;/g;
  s/>/&gt;/g;
  $_;
}

###############################################################################
# Converting text to form string
#
sub hf
{ local($_)=$_[0];
  s/\"/&quod;/g;
  $_;
}

###############################################################################
# Heading of HTML page
#
sub print_html_head
{ local($head)=$_[0];
  local($body)=$_[1];
  local($bgimg)=$bg_image ? " BACKGROUND=\"$bg_image\"" : "";
  print <<EOH;
<HTML><HEAD>
<TITLE>$head</TITLE>
</HEAD>
<BODY BGCOLOR=${bg_paper}${bgimg}>
EOH
  print <<EOH if $body;
<CENTER><FONT SIZE=+1 FACE=Verdana,Arial,Helvetica>
$body
</FONT></CENTER>
EOH
  &print_html_hr if $body;
}

###############################################################################
# Footer of HTML page
#
sub print_html_footer
{ print <<EOH unless $_[0]==1;
<HR>
<TABLE BORDER=0 CELLSPACING=0 CELLPADDING=0 WIDTH=99%><TR><TD ALIGN=RIGHT>
EOH
  print <<EOH;
<FONT SIZE=-2><I>
Served by <A HREF="http://www.amsoft.ru/ruboard/">ruboard</A> %VERSION%;
Copyright &copy; 1998 by Andrew Maltsev.
</I></FONT>
EOH
  print <<EOH unless $_[0]==1;
</TD></TR></TABLE>
</BODY>
</HTML>
EOH
}

############################
# Locking Data Number
#
sub lock_number
{ $lockname="$datafile.lock";
  $locktmp="$datafile.$$";
  open(TMP,">$locktmp") || &error;
  print TMP "$$\n";
  close(TMP);
  $to_sleep=1;
  while(!link($locktmp,$lockname))
   { open(TMP,"<$lockname");
     $pid=<TMP>;
     close(TMP);
     if(!kill 0,$pid)
      { unlink($lockname);
        last;
      }
     sleep($to_sleep++);
   }
  unlink($locktmp);
}

############################
# Unlocking Data Number
#
sub unlock_number
{ unlink("$datafile.lock");
}

###############################################################################
# Finish
1;
